@@ -111,6 +111,11 @@ forms supported by parse_ipv4). Note that scope-id's are not supported
 
 This function works similarly to C<inet_pton AF_INET6, ...>.
 
+Example:
+
+   print unpack "H*", parse_ipv6 "2002:5345::10.0.0.1";
+   # => 2002534500000000000000000a000001
+
 =cut
 
 sub parse_ipv6($) {
@@ -171,6 +176,11 @@ If the C<$text> to parse is a mapped IPv4 in IPv6 address (:ffff::<ipv4>),
 then it will be treated as an IPv4 address. If you don't want that, you
 have to call C<parse_ipv4> and/or C<parse_ipv6> manually.
 
+Example:
+
+   print unpack "H*", parse_address "10.1.2.3";
+   # => 0a010203
+
 =item $ipn = AnyEvent::Socket::aton $ip
 
 Same as C<parse_address>, but not exported (think C<Socket::inet_aton> but
@@ -197,6 +207,10 @@ Works like the builtin function of the same name, except it tries hard to
 work even on broken platforms (well, that's windows), where getprotobyname
 is traditionally very unreliable.
 
+Example: get the protocol number for TCP (usually 6)
+
+   my $proto = getprotobyname "tcp";
+
 =cut
 
 # microsoft can't even get getprotobyname working (the etc/protocols file
@@ -353,6 +367,11 @@ If the C<$ipn> is a mapped IPv4 in IPv6 address (:ffff::<ipv4>), then just
 the contained IPv4 address will be returned. If you do not want that, you
 have to call C<format_ipv6> manually.
 
+Example:
+
+   print format_address "\x01\x02\x03\x05";
+   => 1.2.3.5
+
 =item $text = AnyEvent::Socket::ntoa $ipn
 
 Same as format_address, but not exported (think C<inet_ntoa>).
@@ -413,12 +432,27 @@ sub format_address($) {
 =item inet_aton $name_or_address, $cb->(@addresses)
 
 Works similarly to its Socket counterpart, except that it uses a
-callback. Also, if a host has only an IPv6 address, this might be passed
-to the callback instead (use the length to detect this - 4 for IPv4, 16
-for IPv6).
+callback. Use the length to distinguish between ipv4 and ipv6 (4 octets
+for IPv4, 16 for IPv6), or use C<format_address> to convert it to a more
+readable format.
 
-Unlike the L<Socket> function of the same name, you can get multiple IPv4
-and IPv6 addresses as result (and maybe even other adrdess types).
+Note that C<resolve_sockaddr>, while initially a more complex interface,
+resolves host addresses, IDNs, service names and SRV records and gives you
+an ordered list of socket addresses to try and should be preferred over
+C<inet_aton>.
+
+Example.
+
+   inet_aton "www.google.com", my $cv = AE::cv;
+   say unpack "H*", $_
+      for $cv->recv;
+   # => d155e363
+   # => d155e367 etc.
+
+   inet_aton "ipv6.google.com", my $cv = AE::cv;
+   say unpack "H*", $_
+      for $cv->recv;
+   # => 20014860a00300000000000000000068
 
 =cut
 
@@ -434,15 +468,34 @@ sub inet_aton {
    } else {
       require AnyEvent::DNS;
 
-      # simple, bad suboptimal algorithm
-      AnyEvent::DNS::a ($name, sub {
-         if (@_) {
-            $cb->(map +(parse_ipv4 $_), @_);
-         } else {
-            $cb->();
-            #AnyEvent::DNS::aaaa ($name, $cb); need inet_pton
-         }
-      });
+      my $ipv4 = $AnyEvent::PROTOCOL{ipv4};
+      my $ipv6 = $AnyEvent::PROTOCOL{ipv6};
+
+      my @res;
+
+      my $cv = AE::cv {
+         $cb->(map @$_, reverse @res);
+      };
+
+      $cv->begin;
+
+      if ($ipv4) {
+         $cv->begin;
+         AnyEvent::DNS::a ($name, sub {
+            $res[$ipv4] = [map &parse_ipv4, @_];
+            $cv->end;
+         });
+      };
+
+      if ($ipv6) {
+         $cv->begin;
+         AnyEvent::DNS::aaaa ($name, sub {
+            $res[$ipv6] = [map &parse_ipv6, @_];
+            $cv->end;
+         });
+      };
+
+      $cv->end;
    }
 }
 
@@ -456,7 +509,7 @@ BEGIN {
            : sub { unpack "S" , $_[0] };
 }
 
-# check for broken platforms with extra field in sockaddr structure
+# check for broken platforms with an extra field in sockaddr structure
 # kind of a rfc vs. bsd issue, as usual (ok, normally it's a
 # unix vs. bsd issue, a iso C vs. bsd issue or simply a
 # correctness vs. bsd issue.)
@@ -470,6 +523,12 @@ structure. Handles both IPv4 and IPv6 host addresses, as well as UNIX
 domain sockets (C<$host> == C<unix/> and C<$service> == absolute
 pathname).
 
+Example:
+
+   my $bind = AnyEvent::Socket::pack_sockaddr 43, v195.234.53.120;
+   bind $socket, $bind
+      or die "bind: $!";
+
 =cut
 
 sub pack_sockaddr($$) {
@@ -532,13 +591,14 @@ and sockaddr structures usable to connect to this node and service in a
 protocol-independent way. It works remotely similar to the getaddrinfo
 posix function.
 
-For internet addresses, C<$node> is either an IPv4 or IPv6 address or an
-internet hostname, and C<$service> is either a service name (port name
-from F</etc/services>) or a numerical port number. If both C<$node> and
-C<$service> are names, then SRV records will be consulted to find the real
-service, otherwise they will be used as-is. If you know that the service
-name is not in your services database, then you can specify the service in
-the format C<name=port> (e.g. C<http=80>).
+For internet addresses, C<$node> is either an IPv4 or IPv6 address, an
+internet hostname (DNS domain name or IDN), and C<$service> is either
+a service name (port name from F</etc/services>) or a numerical port
+number. If both C<$node> and C<$service> are names, then SRV records
+will be consulted to find the real service, otherwise they will be
+used as-is. If you know that the service name is not in your services
+database, then you can specify the service in the format C<name=port>
+(e.g. C<http=80>).
 
 For UNIX domain sockets, C<$node> must be the string C<unix/> and
 C<$service> must be the absolute pathname of the socket. In this case,
@@ -608,10 +668,10 @@ sub resolve_sockaddr($$$$$$) {
               or Carp::croak "$service/$proto: service unknown";
    }
 
-   my @target = [$node, $port];
-
    # resolve a records / provide sockaddr structures
    my $resolve = sub {
+      my @target = @_;
+
       my @res;
       my $cv = AE::cv {
          $cb->(
@@ -667,42 +727,46 @@ sub resolve_sockaddr($$$$$$) {
       $cv->end;
    };
 
+   $node = AnyEvent::Util::idn_to_ascii $node
+      if $node =~ /[^\x00-\x7f]/;
+
    # try srv records, if applicable
    if ($node eq "localhost") {
-      @target = (["127.0.0.1", $port], ["::1", $port]);
-      &$resolve;
+      $resolve->(["127.0.0.1", $port], ["::1", $port]);
    } elsif (defined $service && !parse_address $node) {
       AnyEvent::DNS::srv $service, $proto, $node, sub {
          my (@srv) = @_;
 
-         # no srv records, continue traditionally
-         @srv
-            or return &$resolve;
+         if (@srv) {
+            # the only srv record has "." ("" here) => abort
+            $srv[0][2] ne "" || $#srv
+               or return $cb->();
 
-         # the only srv record has "." ("" here) => abort
-         $srv[0][2] ne "" || $#srv
-            or return $cb->();
-
-         # use srv records then
-         @target = map ["$_->[3].", $_->[2]],
-                      grep $_->[3] ne ".",
-                         @srv;
-
-         &$resolve;
+            # use srv records then
+            $resolve->(
+               map ["$_->[3].", $_->[2]],
+                  grep $_->[3] ne ".",
+                     @srv
+            );
+         } else {
+            # no srv records, continue traditionally
+            $resolve->([$node, $port]);
+         }
       };
    } else {
-      &$resolve;
+      # most common case
+      $resolve->([$node, $port]);
    }
 }
 
 =item $guard = tcp_connect $host, $service, $connect_cb[, $prepare_cb]
 
-This is a convenience function that creates a TCP socket and makes a 100%
-non-blocking connect to the given C<$host> (which can be a hostname or
-a textual IP address, or the string C<unix/> for UNIX domain sockets)
-and C<$service> (which can be a numeric port number or a service name,
-or a C<servicename=portnumber> string, or the pathname to a UNIX domain
-socket).
+This is a convenience function that creates a TCP socket and makes a
+100% non-blocking connect to the given C<$host> (which can be a DNS/IDN
+hostname or a textual IP address, or the string C<unix/> for UNIX domain
+sockets) and C<$service> (which can be a numeric port number or a service
+name, or a C<servicename=portnumber> string, or the pathname to a UNIX
+domain socket).
 
 If both C<$host> and C<$port> are names, then this function will use SRV
 records to locate the real target(s).
@@ -818,7 +882,7 @@ Example: connect to a UNIX domain socket.
 sub tcp_connect($$$;$) {
    my ($host, $port, $connect, $prepare) = @_;
 
-   # see http://cr.yp.to/docs/connect.html for some background
+   # see http://cr.yp.to/docs/connect.html for some tricky aspects
    # also http://advogato.org/article/672.html
 
    my %state = ( fh => undef );
@@ -1018,7 +1082,7 @@ sub tcp_server($$$;$) {
 
    $state{aw} = AE::io $state{fh}, 0, sub {
       # this closure keeps $state alive
-      while (my $peer = accept my $fh, $state{fh}) {
+      while ($state{fh} && (my $peer = accept my $fh, $state{fh})) {
          fh_nonblocking $fh, 1; # POSIX requires inheritance, the outside world does not
 
          my ($service, $host) = unpack_sockaddr $peer;
